@@ -2,6 +2,8 @@ import logging
 import datetime
 from typing import Any, Dict, List, Literal, Optional, Set, TYPE_CHECKING, cast, Tuple
 import contextvars
+from uuid import UUID
+
 from langchain_core import language_models
 from langchain_core.tracers import BaseTracer
 from langchain_core.tracers.schemas import Run
@@ -13,6 +15,7 @@ from opik.types import DistributedTraceHeadersDict, ErrorInfoDict
 from opik.validation import parameters_validator
 from . import (
     base_llm_patcher,
+    helpers as langchain_helpers,
     opik_encoder_extension,
     provider_usage_extractors,
 )
@@ -22,10 +25,7 @@ import opik.context_storage as context_storage
 import opik.decorator.tracing_runtime_config as tracing_runtime_config
 
 if TYPE_CHECKING:
-    from uuid import UUID
-
     from langchain_core.runnables.graph import Graph
-
     from langchain_core.messages import BaseMessage
 
 LOGGER = logging.getLogger(__name__)
@@ -89,10 +89,10 @@ class OpikTracer(BaseTracer):
 
         self._trace_default_tags = tags
 
-        self._span_data_map: Dict["UUID", span.SpanData] = {}
+        self._span_data_map: Dict[UUID, span.SpanData] = {}
         """Map from run id to span data."""
 
-        self._created_traces_data_map: Dict["UUID", trace.TraceData] = {}
+        self._created_traces_data_map: Dict[UUID, trace.TraceData] = {}
         """Map from run id to trace data."""
 
         self._created_traces: List[trace.Trace] = []
@@ -114,36 +114,46 @@ class OpikTracer(BaseTracer):
         ] = contextvars.ContextVar("root_run_external_parent_span_id", default=None)
 
     def _is_opik_span_created_by_this_tracer(self, span_id: str) -> bool:
-        return any(span.id == span_id for span in self._span_data_map.values())
+        return any(span_.id == span_id for span_ in self._span_data_map.values())
 
     def _is_opik_trace_created_by_this_tracer(self, trace_id: str) -> bool:
         return any(
-            trace.id == trace_id for trace in self._created_traces_data_map.values()
+            trace_.id == trace_id for trace_ in self._created_traces_data_map.values()
         )
 
-    def _persist_run(self, run: "Run") -> None:
+    def _persist_run(self, run: Run) -> None:
+        if run.id not in self._span_data_map:
+            LOGGER.warning(
+                f"Span data for run '{run.id}' not found in the span data map. Skipping processing of _persist_run."
+            )
+            return
+
+        span_data = self._span_data_map[run.id]
+
         run_dict: Dict[str, Any] = run.dict()
 
         error_info: Optional[ErrorInfoDict]
+        trace_additional_metadata: Dict[str, Any] = {}
+
         if run_dict["error"] is not None:
             output = None
-            error_info = {
-                "exception_type": "Exception",
-                "traceback": run_dict["error"],
-            }
+            error_info = ErrorInfoDict(
+                exception_type="Exception",
+                traceback=run_dict["error"],
+            )
         else:
-            output = run_dict["outputs"]
+            output, trace_additional_metadata = (
+                langchain_helpers.split_big_langgraph_outputs(run_dict["outputs"])
+            )
             error_info = None
 
-        span_data = self._span_data_map[run.id]
-
         if (
             span_data.parent_span_id is not None
             and self._is_opik_span_created_by_this_tracer(span_data.parent_span_id)
         ):
-            # Langchain lost parent-child relationship for Run, so it calls _persist_run
-            # for a subchain when the ACTUAL root run is not yet persisted.
-            # However we know that the parent span was created by this tracer, so we don't
+            # Langchain lost the parent-child relationship for Run, so it calls _persist_run
+            # for a subchain when the ACTUAL root run has not yet persisted.
+            # However, we know that this tracer created the parent span, so we don't
             # want to finalize the trace
             return
 
@@ -156,6 +166,9 @@ class OpikTracer(BaseTracer):
             if trace_data.input == {"input": ""}:
                 trace_data.input = run_dict["inputs"]
 
+            if trace_additional_metadata:
+                trace_data.update(metadata=trace_additional_metadata)
+
             trace_data.init_end_time().update(output=output, error_info=error_info)
             trace_ = self._opik_client.trace(**trace_data.as_parameters)
 
@@ -227,9 +240,6 @@ class OpikTracer(BaseTracer):
             project_name=self._project_name,
             thread_id=self._thread_id,
         )
-
-        self._created_traces_data_map[run_dict["id"]] = trace_data
-
         span_data = span.SpanData(
             trace_id=trace_data.id,
             parent_span_id=None,
@@ -240,9 +250,6 @@ class OpikTracer(BaseTracer):
             tags=self._trace_default_tags,
             project_name=self._project_name,
         )
-
-        self._span_data_map[run_dict["id"]] = span_data
-
         return trace_data, span_data
 
     def _attach_span_to_existing_span(
@@ -266,7 +273,6 @@ class OpikTracer(BaseTracer):
             project_name=project_name,
             type=_get_span_type(run_dict),
         )
-        self._span_data_map[run_dict["id"]] = span_data
         if not self._is_opik_trace_created_by_this_tracer(span_data.trace_id):
             self._externally_created_traces_ids.add(span_data.trace_id)
 
@@ -293,7 +299,6 @@ class OpikTracer(BaseTracer):
             project_name=project_name,
             type=_get_span_type(run_dict),
         )
-        self._span_data_map[run_dict["id"]] = span_data
         if not self._is_opik_trace_created_by_this_tracer(current_trace_data.id):
             self._externally_created_traces_ids.add(current_trace_data.id)
         return span_data
@@ -316,14 +321,18 @@ class OpikTracer(BaseTracer):
             project_name=self._project_name,
             type=_get_span_type(run_dict),
         )
-        self._span_data_map[run_dict["id"]] = span_data
         self._externally_created_traces_ids.add(span_data.trace_id)
         return span_data
 
-    def _process_start_span(self, run: "Run") -> None:
+    def _process_start_span(self, run: Run) -> None:
+        try:
+            self._process_start_span_unsafe(run)
+        except Exception as e:
+            LOGGER.error("Failed during _process_start_span: %s", e, exc_info=True)
+
+    def _process_start_span_unsafe(self, run: Run) -> None:
         run_dict: Dict[str, Any] = run.dict()
         new_span_data: span.SpanData
-        new_trace_data: Optional[trace.TraceData] = None
 
         if not run.parent_run_id:
             # This is the first run for the chain.
@@ -336,6 +345,13 @@ class OpikTracer(BaseTracer):
                 ):
                     self._opik_client.trace(**new_trace_data.as_start_parameters)
 
+            # save new span and trace data to local maps to be able to retrieve them later
+            self._save_span_trace_data_to_local_maps(
+                run_id=run.id,
+                span_data=new_span_data,
+                trace_data=new_trace_data,
+            )
+
             self._opik_context_storage.add_span_data(new_span_data)
             if (
                 self._opik_client.config.log_start_trace_span
@@ -362,7 +378,11 @@ class OpikTracer(BaseTracer):
         )
         new_span_data.update(metadata={"created_from": "langchain"})
 
-        self._span_data_map[run.id] = new_span_data
+        self._save_span_trace_data_to_local_maps(
+            run_id=run.id,
+            span_data=new_span_data,
+            trace_data=None,
+        )
 
         if new_span_data.trace_id not in self._externally_created_traces_ids:
             self._created_traces_data_map[run.id] = self._created_traces_data_map[
@@ -376,10 +396,16 @@ class OpikTracer(BaseTracer):
         ):
             self._opik_client.span(**new_span_data.as_start_parameters)
 
-    def _process_end_span(self, run: "Run") -> None:
+    def _process_end_span(self, run: Run) -> None:
+        span_data = None
         try:
-            run_dict: Dict[str, Any] = run.dict()
+            if run.id not in self._span_data_map:
+                LOGGER.warning(
+                    f"Span data for run '{run.id}' not found in the span data map. Skipping processing of end span."
+                )
+                return
             span_data = self._span_data_map[run.id]
+            run_dict: Dict[str, Any] = run.dict()
 
             usage_info = provider_usage_extractors.try_extract_provider_usage_data(
                 run_dict
@@ -391,8 +417,15 @@ class OpikTracer(BaseTracer):
             if span_data.input == {"input": ""}:
                 span_data.input = run_dict["inputs"]
 
+            filtered_output, additional_metadata = (
+                langchain_helpers.split_big_langgraph_outputs(run_dict["outputs"])
+            )
+
+            if additional_metadata:
+                span_data.update(metadata=additional_metadata)
+
             span_data.init_end_time().update(
-                output=run_dict["outputs"],
+                output=filtered_output,
                 usage=(
                     usage_info.usage.provider_usage.model_dump()
                     if isinstance(usage_info.usage, llm_usage.OpikUsage)
@@ -407,12 +440,20 @@ class OpikTracer(BaseTracer):
         except Exception as e:
             LOGGER.error(f"Failed during _process_end_span: {e}", exc_info=True)
         finally:
-            self._opik_context_storage.trim_span_data_stack_to_certain_span(
-                span_id=span_data.id
-            )
-            self._opik_context_storage.pop_span_data(ensure_id=span_data.id)
+            if span_data is not None:
+                self._opik_context_storage.trim_span_data_stack_to_certain_span(
+                    span_id=span_data.id
+                )
+                self._opik_context_storage.pop_span_data(ensure_id=span_data.id)
 
-    def _process_end_span_with_error(self, run: "Run") -> None:
+    def _process_end_span_with_error(self, run: Run) -> None:
+        if run.id not in self._span_data_map:
+            LOGGER.warning(
+                f"Span data for run '{run.id}' not found in the span data map. Skipping processing of _process_end_span_with_error."
+            )
+            return
+
+        span_data = None
         try:
             run_dict: Dict[str, Any] = run.dict()
             span_data = self._span_data_map[run.id]
@@ -430,10 +471,11 @@ class OpikTracer(BaseTracer):
         except Exception as e:
             LOGGER.debug(f"Failed during _process_end_span_with_error: {e}")
         finally:
-            self._opik_context_storage.trim_span_data_stack_to_certain_span(
-                span_id=span_data.id
-            )
-            self._opik_context_storage.pop_span_data(ensure_id=span_data.id)
+            if span_data is not None:
+                self._opik_context_storage.trim_span_data_stack_to_certain_span(
+                    span_id=span_data.id
+                )
+                self._opik_context_storage.pop_span_data(ensure_id=span_data.id)
 
     def _update_thread_id_from_metadata(self, run_dict: Dict[str, Any]) -> None:
         if not self._thread_id:
@@ -443,6 +485,18 @@ class OpikTracer(BaseTracer):
             if thread_id:
                 self._thread_id = thread_id
 
+    def _save_span_trace_data_to_local_maps(
+        self,
+        run_id: UUID,
+        span_data: Optional[span.SpanData],
+        trace_data: Optional[trace.TraceData],
+    ) -> None:
+        if span_data is not None:
+            self._span_data_map[run_id] = span_data
+
+        if trace_data is not None:
+            self._created_traces_data_map[run_id] = trace_data
+
     def flush(self) -> None:
         """
         Flush to ensure all data is sent to the Opik server.
@@ -464,7 +518,7 @@ class OpikTracer(BaseTracer):
 
         return False
 
-    def _on_llm_start(self, run: "Run") -> None:
+    def _on_llm_start(self, run: Run) -> None:
         """Process the LLM Run upon start."""
         if self._skip_tracking():
             return
@@ -476,13 +530,13 @@ class OpikTracer(BaseTracer):
         serialized: Dict[str, Any],
         messages: List[List["BaseMessage"]],
         *,
-        run_id: "UUID",
+        run_id: UUID,
         tags: Optional[List[str]] = None,
-        parent_run_id: Optional["UUID"] = None,
+        parent_run_id: Optional[UUID] = None,
         metadata: Optional[Dict[str, Any]] = None,
         name: Optional[str] = None,
         **kwargs: Any,
-    ) -> "Run":
+    ) -> Run:
         """Start a trace for an LLM run.
 
         Duplicated from Langchain tracer, it is disabled by default in all tracers, see https://github.com/langchain-ai/langchain/blob/fdda1aaea14b257845a19023e8af5e20140ec9fe/libs/core/langchain_core/callbacks/manager.py#L270-L289 and https://github.com/langchain-ai/langchain/blob/fdda1aaea14b257845a19023e8af5e20140ec9fe/libs/core/langchain_core/tracers/core.py#L168-L180
@@ -524,63 +578,63 @@ class OpikTracer(BaseTracer):
         self._on_chat_model_start(chat_model_run)
         return chat_model_run
 
-    def _on_chat_model_start(self, run: "Run") -> None:
+    def _on_chat_model_start(self, run: Run) -> None:
         """Process the Chat Model Run upon start."""
         if self._skip_tracking():
             return
 
         self._process_start_span(run)
 
-    def _on_llm_end(self, run: "Run") -> None:
+    def _on_llm_end(self, run: Run) -> None:
         """Process the LLM Run."""
         if self._skip_tracking():
             return
 
         self._process_end_span(run)
 
-    def _on_llm_error(self, run: "Run") -> None:
+    def _on_llm_error(self, run: Run) -> None:
         """Process the LLM Run upon error."""
         if self._skip_tracking():
             return
 
         self._process_end_span_with_error(run)
 
-    def _on_chain_start(self, run: "Run") -> None:
+    def _on_chain_start(self, run: Run) -> None:
         """Process the Chain Run upon start."""
         if self._skip_tracking():
             return
 
         self._process_start_span(run)
 
-    def _on_chain_end(self, run: "Run") -> None:
+    def _on_chain_end(self, run: Run) -> None:
         """Process the Chain Run."""
         if self._skip_tracking():
             return
 
         self._process_end_span(run)
 
-    def _on_chain_error(self, run: "Run") -> None:
+    def _on_chain_error(self, run: Run) -> None:
         """Process the Chain Run upon error."""
         if self._skip_tracking():
             return
 
         self._process_end_span_with_error(run)
 
-    def _on_tool_start(self, run: "Run") -> None:
+    def _on_tool_start(self, run: Run) -> None:
         """Process the Tool Run upon start."""
         if self._skip_tracking():
             return
 
         self._process_start_span(run)
 
-    def _on_tool_end(self, run: "Run") -> None:
+    def _on_tool_end(self, run: Run) -> None:
         """Process the Tool Run."""
         if self._skip_tracking():
             return
 
         self._process_end_span(run)
 
-    def _on_tool_error(self, run: "Run") -> None:
+    def _on_tool_error(self, run: Run) -> None:
         """Process the Tool Run upon error."""
         if self._skip_tracking():
             return
